<p align="center">
    <a href="https://sylius.com" target="_blank">
        <img src="https://www.dedi-agency.com/wp-content/uploads/2014/05/Dedi_logo_HD.png" />
    </a>
</p>

<h1 align="center">Plugin Dedi Voucher</h1>

## Documentation

For a comprehensive guide on Sylius Plugins development please go to Sylius documentation,
there you will find the <a href="https://docs.sylius.com/en/latest/plugin-development-guide/index.html">Plugin Development Guide</a>, that is full of examples.

## Quickstart Installation

```bash
$ make install-plugin
```

Open your browser and go to `http://localhost:9000`.

## Usage

### Running plugin tests

  - PHPUnit

    ```bash
    $ vendor/bin/phpunit
    ```

  - PHPSpec

    ```bash
    $ vendor/bin/phpspec run
    ```

  - Behat (non-JS scenarios)

    ```bash
    $ vendor/bin/behat --tags="~@javascript"
    ```

  - Behat (JS scenarios)
 
    1. Download [Chromedriver](https://sites.google.com/a/chromium.org/chromedriver/)
    
    2. Download [Selenium Standalone Server](https://www.seleniumhq.org/download/).
    
    2. Run Selenium server with previously downloaded Chromedriver:
    
        ```bash
        $ java -Dwebdriver.chrome.driver=chromedriver -jar selenium-server-standalone.jar
        ```
        
    3. Run test application's webserver on `localhost:8080`:
    
        ```bash
        $ (cd tests/Application && bin/console server:run localhost:8080 -d public -e test)
        ```
    
    4. Run Behat:
    
        ```bash
        $ vendor/bin/behat --tags="@javascript"
        ```

### Opening Sylius with your plugin

After installation and with docker containers running, go to `http://0.0.0.0:9000/`

## Installation on a Project

Require plugin with composer:

    composer require dedi/sylius-voucher-plugin

Import configuration:


```yaml
# config/packages/dedi_sylius_voucher.yaml

imports:
    # 
    - { resource: "@DediSyliusVoucherPlugin/Resources/config/app/config.yaml" }
```

(Optional) Import fixtures

If you wish to have some vouchers to play with in your application during development.

```yaml
# config/packages/dedi_sylius_voucher.yaml
imports:
    # ...
    - { resource: "@DediSyliusVoucherPlugin/Resources/config/app/fixtures.yaml" }
```

Import routing:

```yaml
# config/routes.yaml
dedi_sylius_voucher:
    resource: "@DediSyliusVoucherPlugin/Resources/config/routes.yaml"
```
or if your app doesn't use locales:

```yaml
# config/routes.yaml
dedi_sylius_voucher:
    resource: "@DediSyliusVoucherPlugin/Resources/config/routes_no_locale.yaml"
```
Add plugin class to your bundles.php:

Make sure you add it before SyliusGridBundle, otherwise you'll get You have requested a non-existent parameter "dedi_sylius_voucher.model.voucher.class". exception.

```php
<?php
$bundles = [
    // ...
    Dedi\SyliusVoucherPlugin\DediSyliusVoucherPlugin::class => ['all' => true],
    Sylius\Bundle\GridBundle\SyliusGridBundle::class => ['all' => true],
    // ...
];
```

Copy templates

You will find the templates you need to override in the test application.
Extend Product, Order, OrderRepository, and CustomerRepository

Extend Product

```php
<?php

# src/Entity/Product/Product.php

declare(strict_types=1);
namespace App\Entity\Product;

use Doctrine\ORM\Mapping as ORM;
use Dedi\SyliusVoucherPlugin\Model\ProductInterface as DediSyliusvoucherProductInterface;
use Dedi\SyliusVoucherPlugin\Model\ProductTrait as DediSyliusvoucherProductTrait;
use Sylius\Component\Core\Model\Product as BaseProduct;

/**
 * @ORM\Entity
 * @ORM\Table(name="sylius_product")
 */
class Product extends BaseProduct implements DediSyliusvoucherProductInterface
{
    use DediSyliusvoucherProductTrait;
}
```

Extend Order

```php
<?php

# src/Entity/Order/Order.php

declare(strict_types=1);
namespace App\Entity\Order;

use Dedi\SyliusVoucherPlugin\Model\OrderInterface as DediSyliusVoucherPluginOrderInterface;
use Dedi\SyliusVoucherPlugin\Model\OrderTrait as DediSyliusVoucherPluginOrderTrait;
use Sylius\Component\Core\Model\Order as BaseOrder;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\Table(name="sylius_order")
 */
class Order extends BaseOrder implements DediSyliusVoucherPluginOrderInterface
{
    use DediSyliusVoucherPluginOrderTrait {
        DediSyliusVoucherPluginOrderTrait::__construct as private __voucherTraitConstruct;
    }
    
    public function __construct()
    {
        $this->__voucherTraitConstruct();

        parent::__construct();
    }
}
```

Extend OrderRepository:

```php
<?php

# src/Doctrine/ORM/OrderRepository.php

declare(strict_types=1);
namespace App\Doctrine\ORM;

use Dedi\SyliusVoucherPlugin\Repository\OrderRepositoryInterface as DediSyliusVoucherPluginOrderRepositoryInterface;
use Dedi\SyliusVoucherPlugin\Doctrine\ORM\OrderRepositoryTrait as DediSyliusVoucherPluginOrderRepositoryTrait;
use Sylius\Bundle\CoreBundle\Doctrine\ORM\OrderRepository as BaseOrderRepository;

class OrderRepository extends BaseOrderRepository implements DediSyliusVoucherPluginOrderRepositoryInterface
{
    use DediSyliusVoucherPluginOrderRepositoryTrait;
}
```

Extend CustomerRepository:

```php
<?php

# src/Doctrine/ORM/CustomerRepository.php

declare(strict_types=1);
namespace App\Doctrine\ORM;

use Dedi\SyliusVoucherPlugin\Repository\CustomerRepositoryInterface as DediSyliusVoucherPluginCustomerRepositoryInterface;
use Dedi\SyliusVoucherPlugin\Doctrine\ORM\CustomerRepositoryTrait as DediSyliusVoucherPluginCustomerRepositoryTrait;
use Sylius\Bundle\CoreBundle\Doctrine\ORM\CustomerRepository as BaseCustomerRepository;

class CustomerRepository extends BaseCustomerRepository implements DediSyliusVoucherPluginCustomerRepositoryInterface
{
    use DediSyliusVoucherPluginCustomerRepositoryTrait;
}
```

Add configuration:

```yaml
# config/services.yaml
sylius_customer:
    resources:
       customer:
            classes:
                repository: App\Doctrine\ORM\CustomerRepository

sylius_order:
    resources:
        order:
            classes:
                model: App\Entity\Order\Order
                repository: App\Doctrine\ORM\OrderRepository
                
sylius_product:
    resources:
        product:
            classes:
                model: App\Entity\Product\Product
```
